@@ -10,7 +10,7 @@ use copypasta::{ClipboardContext, ClipboardProvider};
 use egui::{
     emath::{pos2, vec2},
     epaint::ClippedShape,
-    CtxRef, Key, Pos2,
+    Context, Key, Pos2,
 };
 use winit::{
     dpi::PhysicalSize,
@@ -34,7 +34,7 @@ pub struct PlatformDescriptor {
 }
 
 #[cfg(feature = "webbrowser")]
-fn handle_links(output: &egui::Output) {
+fn handle_links(output: &egui::PlatformOutput) {
     if let Some(open_url) = &output.open_url {
         // This does not handle open_url.new_tab
         // webbrowser does not support web anyway
@@ -45,7 +45,7 @@ fn handle_links(output: &egui::Output) {
 }
 
 #[cfg(feature = "clipboard")]
-fn handle_clipboard(output: &egui::Output, clipboard: Option<&mut ClipboardContext>) {
+fn handle_clipboard(output: &egui::PlatformOutput, clipboard: Option<&mut ClipboardContext>) {
     if !output.copied_text.is_empty() {
         if let Some(clipboard) = clipboard {
             if let Err(err) = clipboard.set_contents(output.copied_text.clone()) {
@@ -58,7 +58,7 @@ fn handle_clipboard(output: &egui::Output, clipboard: Option<&mut ClipboardConte
 /// Provides the integration between egui and winit.
 pub struct Platform {
     scale_factor: f64,
-    context: CtxRef,
+    context: Context,
     raw_input: egui::RawInput,
     modifier_state: ModifiersState,
     pointer_pos: Option<egui::Pos2>,
@@ -70,7 +70,7 @@ pub struct Platform {
 impl Platform {
     /// Creates a new `Platform`.
     pub fn new(descriptor: PlatformDescriptor) -> Self {
-        let context = CtxRef::default();
+        let context = Context::default();
 
         context.set_fonts(descriptor.font_definitions.clone());
         context.set_style(descriptor.style);
@@ -154,6 +154,22 @@ impl Platform {
                         }
                     }
                 }
+                Touch(touch) => {
+                    let pointer_pos = pos2(
+                        touch.location.x as f32 / self.scale_factor as f32,
+                        touch.location.y as f32 / self.scale_factor as f32,
+                    );
+
+                    self.raw_input.events.push(egui::Event::PointerButton {
+                        pos: pointer_pos,
+                        button: egui::PointerButton::Primary,
+                        pressed: match touch.phase {
+                            winit::event::TouchPhase::Started => true,
+                            _ => false,
+                        },
+                        modifiers: Default::default(),
+                    });
+                }
                 MouseWheel { delta, .. } => {
                     let mut delta = match delta {
                         winit::event::MouseScrollDelta::LineDelta(x, y) => {
@@ -261,6 +277,8 @@ impl Platform {
 
                 CursorMoved { .. } => self.context().is_using_pointer(),
 
+                Touch { .. } => self.context().is_using_pointer(),
+
                 _ => false,
             },
 
@@ -284,13 +302,13 @@ impl Platform {
     pub fn end_frame(
         &mut self,
         window: Option<&winit::window::Window>,
-    ) -> (egui::Output, Vec<ClippedShape>) {
+    ) -> egui::FullOutput {
         // otherwise the below line gets flagged by clippy if both clipboard and webbrowser features are disabled
         #[allow(clippy::let_and_return)]
-        let parts = self.context.end_frame();
+        let output = self.context.end_frame();
 
         if let Some(window) = window {
-            if let Some(cursor_icon) = egui_to_winit_cursor_icon(parts.0.cursor_icon) {
+            if let Some(cursor_icon) = egui_to_winit_cursor_icon(output.platform_output.cursor_icon) {
                 window.set_cursor_visible(true);
                 // if the pointer is located inside the window, set cursor icon
                 if self.pointer_pos.is_some() {
@@ -302,18 +320,24 @@ impl Platform {
         }
 
         #[cfg(feature = "clipboard")]
-        handle_clipboard(&parts.0, self.clipboard.as_mut());
+        handle_clipboard(&output.platform_output, self.clipboard.as_mut());
 
         #[cfg(feature = "webbrowser")]
-        handle_links(&parts.0);
+        handle_links(&output.platform_output);
 
-        parts
+        output
     }
 
     /// Returns the internal egui context.
-    pub fn context(&self) -> CtxRef {
+    pub fn context(&self) -> Context {
         self.context.clone()
     }
+
+    /// Returns a mutable reference to the raw input that will be passed to egui
+    /// the next time [`Self::begin_frame`] is called
+    pub fn raw_input_mut(&mut self) -> &mut egui::RawInput {
+        &mut self.raw_input
+    }
 }
 
 /// Translates winit to egui keycodes.
